LIBRARY			CSPICE
EXPORTS			appndc_c
				appndd_c
				appndi_c
				axisar_c
				azlcpo_c
				azlrec_c
				b1900_c
				b1950_c
				badkpv_c
				bltfrm_c
				bodc2n_c
				bodc2s_c
				boddef_c
				bodeul_
				bodfnd_c
				bodn2c_c
				bods2c_c
				bodvar_c
				bodvcd_c
				bodvrd_c
				brcktd_c
				brckti_c
				bschoc_c
				bschoi_c
				bsrchc_c
				bsrchd_c
				bsrchi_c
				card_c
				ccifrm_c
				cgv2el_c
				chbder_c
				chbigr_c
				chbint_c
				chbval_c
				chkin_c
				chkout_c
				cidfrm_c
				ckcls_c
				ckcov_c
				ckfrot_
				ckfrot_c
				ckfxfm_c
				ckgp_c
				ckgpav_c
				ckgr02_c
				ckgr03_c
				cklpf_c
				ckmeta_c
				cknr02_c
				cknr03_c
				ckobj_c
				ckopn_c
				ckupf_c
				ckw01_c
				ckw02_c
				ckw03_c
				ckw05_c
				clearc_c
				cleard_c
				cleari_c
				clight_c
				clpool_c
				cltext_
				cmprss_c
				cnmfrm_c
				conics_c
				convrt_c
				copy_c
				cpos_c
				cposr_c
				cvpool_c
				cyllat_c
				cylrec_c
				cylsph_c
				dafac_c
				dafbbs_c
				dafbfs_c
				dafcls_c
				dafcs_c
				dafdc_c
				dafec_c
				daffna_c
				daffpa_c
				dafgda_c
				dafgh_c
				dafgn_c
				dafgs_c
				dafgsr_c
				dafhsf_c
				dafopr_c
				dafopw_c
				dafps_c
				dafrda_c
				dafrfr_c
				dafrs_c
				dafus_c
				dasac_c
				dasadc_c
				dasadd_c
				dasadi_c
				dascls_c
				dasdc_c
				dasec_c
				dashfn_c
				dashfs_c
				daslla_c
				dasllc_c
				dasonw_
				dasonw_c
				dasopr_c
				dasops_c
				dasopw_c
				dasrdc_c
				dasrdd_c
				dasrdi_c
				dasrfr_c
				dasudc_c
				dasudd_c
				dasudi_c
				daswbr_c
				dazldr_c
				dcyldr_c
				deltet_c
				det_c
				dgeodr_c
				diags2_c
				diff_c
				dlabbs_c
				dlabfs_c
				dlabns_c
				dlaens_c
				dlafns_c
				dlafps_c
				dlaopn_c
				dlatdr_c
				dnearp_c
				dp2hx_c
				dpgrdr_c
				dpmax_c
				dpmin_c
				dpr_c
				drdazl_c
				drdcyl_c
				drdgeo_c
				drdlat_c
				drdpgr_c
				drdsph_c
				dskb02_c
				dskcls_c
				dskd02_c
				dskgd_c
				dskgtl_c
				dski02_c
				dskmi2_c
				dskn02_c
				dskobj_c
				dskopn_c
				dskp02_c
				dskrb2_c
				dsksrf_c
				dskstl_c
				dskv02_c
				dskw02_c
				dskx02_c
				dskxsi_c
				dskxv_c
				dskz02_c
				dsphdr_c
				dtpool_c
				ducrss_c
				dvcrss_c
				dvdot_c
				dvhat_c
				dvnorm_c
				dvpool_c
				dvsep_c
				edlimb_c
				ednmpt_c
				edpnt_c 
				edterm_c
				ekacec_c
				ekaced_c
				ekacei_c
				ekaclc_c
				ekacld_c
				ekacli_c
				ekappr_c
				ekbseg_c
				ekccnt_c
				ekcii_c
				ekcls_c
				ekdelr_c
				ekffld_c
				ekfind_c
				ekgc_c
				ekgd_c
				ekgi_c
				ekifld_c
				ekinsr_c
				eklef_c
				eknelt_c
				eknseg_c
				ekntab_c
				ekopn_c
				ekopr_c
				ekops_c
				ekopw_c
				ekpsel_c
				ekrcec_c
				ekrced_c
				ekrcei_c
				ekssum_c
				ektnam_c
				ekucec_c
				ekuced_c
				ekucei_c
				ekuef_c
				el2cgv_c
				elemc_c
				elemd_c
				elemi_c
				eqncpv_c
				eqstr_c
				erract_c
				errch_c
				errdev_c
				errdp_c
				errint_c
				errprt_c
				esrchc_c
				et2lst_c
				et2utc_c
				etcal_c
				eul2m_c
				eul2xf_c
				ev2lin_
				evsgp4_c
				exists_c
				expool_c
				failed_c
				filld_c 
				filli_c
				fn2lun_
				fovray_c
				fovtrg_c
				frame_c
				frinfo_c
				frmnam_c
				ftncls_c
				furnsh_c
				gcpool_c
				gdpool_c
				georec_c
				getcml_c
				getelm_c
				getfat_c
				getfov_c
				getfvn_c
				getmsg_c
				gfbail_c
				gfclrh_c
				gfdist_c
				gfevnt_c
				gffove_c
				gfilum_c
				gfinth_c
				gfocce_c
				gfoclt_c
				gfpa_c
				gfposc_c
				gfrefn_c
				gfrepf_c
				gfrepi_c
				gfrepu_c
				gfrfov_c
				gfrr_c
				gfsep_c
				gfsntc_c
				gfsstp_c
				gfstep_c
				gfstol_c
				gfsubc_c
				gftfov_c
				gfudb_c
				gfuds_c
				gipool_c
				gnpool_c
				halfpi_c
				hrmesp_c
				hrmint_c
				hx2dp_c
				ident_c
				illum_c
				illumf_c
				illumg_c
				ilumin_c
				inedpl_c
				inelpl_c
				inrypl_c
				insrtc_c
				insrtd_c
				insrti_c
				inter_c
				intmax_c
				intmin_c
				invert_c
				invort_c
				invstm_c
				irfnam_
				irfnum_
				irfrot_
				irftrn_
				isordv_c
				isrchc_c
				isrchd_c
				isrchi_c
				isrot_c
				iswhsp_c
				j1900_c
				j1950_c
				j2000_c
				j2100_c
				jyear_c
				kclear_c
				kdata_c
				kepleq_
				kinfo_c
				kplfrm_c
				kpsolv_
				ktotal_c
				kxtrct_c
				lastnb_c
				latcyl_c
				latrec_c
				latsph_c
				latsrf_c
				lcase_c
				ldpool_c
				lgresp_c
				lgrind_c
				lgrint_c
				limbpt_c
				lmpool_c
				lparse_c
				lparsm_c
				lparss_c
				lspcn_c
				lstlec_c
				lstled_c
				lstlei_c
				lstltc_c
				lstltd_c
				lstlti_c
				ltime_c
				lx4dec_c
				lx4num_c
				lx4sgn_c
				lx4uns_c
				lxqstr_c
				m2eul_c
				m2q_c
				matchi_c
				matchw_c
				maxd_c
				maxi_c
				mequ_c
				mequg_c
				mind_c
				mini_c
				moved_c 
				mtxm_c
				mtxmg_c
				mtxv_c
				mtxvg_c
				mxm_c
				mxmg_c
				mxmt_c
				mxmtg_c
				mxv_c
				mxvg_c
				namfrm_c
				ncpos_c
				ncposr_c
				nearpt_c
				nextwd_c
				npedln_c
				npelpt_c
				nplnpt_c
				nthwd_c 
				nvc2pl_c
				nvp2pl_c
				occult_c
				ordc_c
				ordd_c
				orderc_c
				orderd_c
				orderi_c
				ordi_c
				oscelt_c
				oscltx_c
				pckcls_c
				pckcov_c
				pckfrm_c
				pcklof_c
				pckopn_c
				pckuof_c
				pckw02_c
				pcpool_c
				pdpool_c
				pgrrec_c
				phaseq_c
				pi_c
				pipool_c
				pjelpl_c
				pl2nvc_c
				pl2nvp_c
				pl2psv_c
				pltar_c
				pltexp_c
				pltnp_c
				pltnrm_c
				pltvol_c
				polyds_c
				pos_c
				posr_c
				prompt_c
				prop2b_c
				prsdp_c
				prsint_c
				psv2pl_c
				putcml_c
				pxform_c
				pxfrm2_c
				q2m_c
				qcktrc_c
				qderiv_c
				qdq2av_c
				qxq_c
				radrec_c
				rav2xf_c
				raxisa_c
				rdtext_c
				recazl_c
				reccyl_c
				recgeo_c
				reclat_c
				recpgr_c
				recrad_c
				recsph_c
				removc_c
				removd_c
				removi_c
				reordc_c
				reordd_c
				reordi_c
				reordl_c
				repmc_c
				repmct_c
				repmd_c
				repmf_c
				repmi_c
				repmot_c
				reset_c
				return_c
				rotate_c
				rotmat_c
				rotvec_c
				rpd_c
				rquad_c
				saelgv_c
				scard_c
				scdecd_c
				sce2c_c
				sce2s_c
				sce2t_c
				scencd_c
				scfmt_c
				scpart_c
				scs2e_c
				sct2e_c
				sctiks_c
				sdiff_c
				set_c
				setmsg_c
				shellc_c
				shelld_c
				shelli_c
				sigerr_c
				sincpt_c
				size_c
				spd_c
				sphcyl_c
				sphlat_c
				sphrec_c
				spk14a_c
				spk14b_c
				spk14e_c
				spkacs_c
				spkapo_c
				spkapp_c
				spkaps_c
				spkcls_c
				spkcov_c
				spkcpo_c
				spkcpt_c
				spkcvo_c
				spkcvt_c
				spkez_c
				spkezp_c
				spkezr_c
				spkgeo_c
				spkgps_c
				spklef_c
				spkltc_c
				spkobj_c
				spkopa_c
				spkopn_c
				spkpds_c
				spkpos_c
				spkpvn_c
				spksfs_c
				spkssb_c
				spksub_c
				spkuds_c
				spkuef_c
				spkw02_c
				spkw03_c
				spkw05_c
				spkw08_c
				spkw09_c
				spkw10_c
				spkw12_c
				spkw13_c
				spkw15_c
				spkw17_c
				spkw18_c
				spkw20_c
				srfc2s_c
				srfcss_c
				srfnrm_c
				srfrec_c
				srfs2c_c
				srfscc_c
				srfxpt_c
				ssize_c
				stelab_c
				stlabx_c
				stpool_c
				str2et_c
				subpnt_c
				subpt_c
				subslr_c
				subsol_c
				sumad_c
				sumai_c
				surfnm_c
				surfpt_c
				surfpv_c
				swpool_c
				sxform_c
				szpool_c
				tangpt_c
				termpt_c
				timdef_c
				timout_c
				tipbod_c
				tisbod_c
				tkfram_
				tkfram_c
				tkvrsn_c
				tparch_c
				tparse_c
				tpictr_c
				trace_c
				trcdep_c
				trcnam_c
				trcoff_c
				trgsep_c
				tsetyr_c
				twopi_c
				twovec_c
				twovxf_c
				txtopn_
				tyear_c
				ucase_c
				ucrss_c
				uddc_c
				uddf_c
				udf_c
				union_c
				unitim_c
				unload_c
				unorm_c
				unormg_c
				utc2et_c
				vadd_c
				vaddg_c
				valid_c
				vcrss_c
				vdist_c
				vdistg_c
				vdot_c
				vdotg_c
				vequ_c
				vequg_c
				vhat_c
				vhatg_c
				vlcom3_c
				vlcom_c
				vlcomg_c
				vminug_c
				vminus_c
				vnorm_c
				vnormg_c
				vpack_c
				vperp_c
				vprjp_c
				vprjpi_c
				vproj_c
				vprojg_c
				vrel_c
				vrelg_c
				vrotv_c
				vscl_c
				vsclg_c
				vsep_c
				vsepg_c
				vsub_c
				vsubg_c
				vtmv_c
				vtmvg_c
				vupack_c
				vzero_c
				vzerog_c
				wncard_c
				wncomd_c
				wncond_c
				wndifd_c
				wnelmd_c
				wnexpd_c
				wnextd_c
				wnfetd_c
				wnfild_c
				wnfltd_c
				wnincd_c
				wninsd_c
				wnintd_c
				wnreld_c
				wnsumd_c
				wnunid_c
				wnvald_c
				writln_
				xf2eul_c
				xf2rav_c
				xfmsta_c
				xpose6_c
				xpose_c
				xposeg_c
				zzdynrot_
				zzgetcml_c
				zzgfsavh_c
